import { describe, expect, it } from "vitest"
import {
  fromLocaleToWikipediaLocale,
  fromSanitizedPageTitleToPageTitle,
  getWikipediaLink,
  sanitizePageTitle,
} from "../wikipedia-utils.ts"

describe("fromLocaleToWikipediaLocale", () => {
  it("should return the correct Wikipedia locale", () => {
    // Arrange - Given
    const input = "en-US"

    // Act - When
    const output = fromLocaleToWikipediaLocale(input)

    // Assert - Then
    const expected = "en"
    expect(output).toEqual(expected)
  })
})

describe("getWikipediaLink", () => {
  it("should return the correct Wikipedia link for the given locale", () => {
    // Arrange - Given
    const input = "en"

    // Act - When
    const output = getWikipediaLink(input)

    // Assert - Then
    const expected = "https://en.wikipedia.org"
    expect(output).toEqual(expected)
  })
})

describe("sanitizePageTitle", () => {
  it("should return the correct sanitized page title", () => {
    // Arrange - Given
    const input = "foo bar"

    // Act - When
    const output = sanitizePageTitle(input)

    // Assert - Then
    const expected = "Foo_bar"
    expect(output).toEqual(expected)
  })

  it("should preserve the characters case", () => {
    // Arrange - Given
    const input = "Foo Bar"

    // Act - When
    const output = sanitizePageTitle(input)

    // Assert - Then
    const expected = "Foo_Bar"
    expect(output).toEqual(expected)
  })

  it("should remove leading and trailing spaces/underscores (rule 1)", () => {
    // Arrange - Given
    const input = "  Abc_def__"

    // Act - When
    const output = sanitizePageTitle(input)

    // Assert - Then
    const expected = "Abc_def"
    expect(output).toEqual(expected)
  })

  it("should reduce consecutive spaces/underscores to a single one (rule 2)", () => {
    // Arrange - Given
    const input = "Abc  def"

    // Act - When
    const output = sanitizePageTitle(input)

    // Assert - Then
    const expected = "Abc_def"
    expect(output).toEqual(expected)
  })

  it("should replace spaces by underscores (rule 3)", () => {
    // Arrange - Given
    const input = "Abc def"

    // Act - When
    const output = sanitizePageTitle(input)

    // Assert - Then
    const expected = "Abc_def"
    expect(output).toEqual(expected)
  })

  it("should capitalize the first character (rule 4)", () => {
    // Arrange - Given
    const input = "abc_def"

    // Act - When
    const output = sanitizePageTitle(input)

    // Assert - Then
    const expected = "Abc_def"
    expect(output).toEqual(expected)
  })

  it("should have a maximum of 255 characters (rule 5)", () => {
    // Arrange - Given
    const input = "a".repeat(256)

    // Act - When
    const output = sanitizePageTitle(input)

    // Assert - Then
    const expected = "A" + "a".repeat(254)
    expect(output).toEqual(expected)
  })
})

describe("fromSanitizedPageTitleToPageTitle", () => {
  it("should return the correct page title", () => {
    // Arrange - Given
    const input = "Foo_bar"

    // Act - When
    const output = fromSanitizedPageTitleToPageTitle(input)

    // Assert - Then
    const expected = "Foo bar"
    expect(output).toEqual(expected)
  })
})
