import ky from "ky"
import { getWikipediaLink, type WikipediaLocale } from "./wikipedia-utils.ts"

/**
 * @see https://www.mediawiki.org/wiki/Wikimedia_REST_API#Terms_and_conditions
 * To avoid impacting other API users, limit your clients to no more than 200 requests/sec to this API overall. Many entry points additionally specify and enforce more restrictive rate limits (HTTP 429 error).
 */

interface WikipediaQueryLinksResponse {
  continue?: {
    plcontinue: string
    continue: string
  }
  query: {
    pages: {
      [key: string]: {
        pageid: number
        ns: number
        title: string
        links: [
          {
            ns: number
            title: string
          },
        ]
      }
    }
  }
  limits: {
    links: number
  }
}

interface GetWikipediaPageInternalLinksInput {
  title: string
  locale: WikipediaLocale
}

interface GetWikipediaPageInternalLinksOutput {
  /**
   * Title of the Wikipedia page.
   */
  title: string

  /**
   * Page id is unique for each page on Wikipedia, can be used to link to the page.
   * @example `https://${locale}.wikipedia.org/?curid=${pageId}`
   */
  pageId: number

  /**
   * List of internal links on the Wikipedia page.
   */
  links: string[]
}

/**
 * Get internal links from a Wikipedia page.
 * @param input
 * @returns
 */
export const getWikipediaPageInternalLinks = async (
  input: GetWikipediaPageInternalLinksInput,
): Promise<GetWikipediaPageInternalLinksOutput> => {
  const links: string[] = []
  let title = input.title
  let pageId = 0
  let plcontinue: string | null = null

  const fetchLinks = async (): Promise<WikipediaQueryLinksResponse> => {
    const url = new URL("/w/api.php", getWikipediaLink(input.locale))
    url.searchParams.append("action", "query")
    url.searchParams.append("titles", title)
    url.searchParams.append("prop", "links")
    url.searchParams.append("pllimit", "max")
    url.searchParams.append("format", "json")
    url.searchParams.append("origin", "*")
    if (plcontinue != null) {
      url.searchParams.set("plcontinue", plcontinue)
    }
    return await ky
      .get(url, {
        method: "GET",
      })
      .json()
  }

  do {
    try {
      const response = await fetchLinks()
      plcontinue = response?.continue?.plcontinue ?? null
      const pages = Object.keys(response.query.pages)
      const page = pages[0] ?? ""
      if (page === "-1" || page === "") {
        break
      }
      const pageData = response.query.pages[page]
      if (pageData == null) {
        break
      }
      title = pageData.title
      pageId = pageData.pageid
      links.push(
        ...pageData.links.map((link) => {
          return link.title
        }),
      )
    } catch (error) {
      console.error("Error", error)
      console.error("title", title)
      throw error
    }
  } while (plcontinue != null)

  return {
    title,
    pageId,
    links,
  }
}

export interface WikipediaPagesInternalLinks {
  [key: string]: GetWikipediaPageInternalLinksOutput
}

export interface GetDeepWikipediaPageInternalLinksInput {
  locale: WikipediaLocale
  data: WikipediaPagesInternalLinks
}

export const getDeepWikipediaPageInternalLinks = async (
  input: GetDeepWikipediaPageInternalLinksInput,
): Promise<void> => {
  const pagesTitles = Object.keys(input.data)
  for (const pageTitle of pagesTitles) {
    const links = input.data[pageTitle]?.links ?? []
    for (const pageTitleLink of links) {
      if (pageTitleLink in input.data) {
        continue
      }
      input.data[pageTitleLink] = await getWikipediaPageInternalLinks({
        locale: input.locale,
        title: pageTitleLink,
      })
      // await getDeepWikipediaPageInternalLinks({
      //   locale: input.locale,
      //   data: input.data,
      // })
    }
  }

  // await Promise.all(
  //   pagesTitles.map(async (pageTitle) => {
  //     const links = input.data[pageTitle]?.links ?? []
  //     await Promise.all(
  //       links.map(async (pageTitleLink) => {
  //         if (pageTitleLink in input.data) {
  //           return
  //         }
  //         input.data[pageTitleLink] = await getWikipediaPageInternalLinks({
  //           locale: input.locale,
  //           title: pageTitleLink,
  //         })
  //         await getDeepWikipediaPageInternalLinks({
  //           locale: input.locale,
  //           data: input.data,
  //         })
  //       }),
  //     )
  //   }),
  // )
}
