import type { Locale } from "@repo/i18n/config"
import {
  capitalize,
  reduceConsecutiveCharacters,
  trimAny,
} from "@repo/utils/strings"

export const WIKIPEDIA_LOCALES = ["en", "fr"] as const
export type WikipediaLocale = (typeof WIKIPEDIA_LOCALES)[number]

const WIKIPEDIA_LOCALES_MAP: Record<Locale, WikipediaLocale> = {
  "en-US": "en",
  "fr-FR": "fr",
}

export const fromLocaleToWikipediaLocale = (
  locale: Locale,
): WikipediaLocale => {
  return WIKIPEDIA_LOCALES_MAP[locale]
}

export const getWikipediaLink = (locale: WikipediaLocale): string => {
  return `https://${locale}.wikipedia.org`
}

/**
 * Converts a page title to a sanitized version (also called "canonical form") that can be used in a URL.
 *
 * The sanitized page title is stored as text with the following restrictions:
 * 1. leading and trailing spaces (` `) and underscores (`_`) are removed.
 * 2. consecutive spaces/underscores are reduced to a single one.
 * 3. spaces are replaced by underscores.
 * 4. first character is capitalized.
 * 5. maximum of 255 characters.
 * @param pageTitle
 * @see https://www.mediawiki.org/wiki/Manual:Page_title
 * @see https://en.wikipedia.org/wiki/Wikipedia:Naming_conventions_(technical_restrictions)
 * @returns
 * @example sanitizePageTitle("foo bar") // "Foo_bar"
 * @example sanitizePageTitle("Foo Bar") // "Foo_Bar"
 */
export const sanitizePageTitle = (pageTitle: string): string => {
  const rule1 = trimAny(pageTitle, [" ", "_"])
  const rule2 = reduceConsecutiveCharacters(rule1, [" ", "_"])
  const rule3 = rule2.replaceAll(" ", "_")
  const rule4 = capitalize(rule3)
  const rule5 = rule4.slice(0, 255)
  return rule5
}

/**
 * Converts a sanitized page title to a page title.
 *
 * A page title is the title of a wiki page, which is a human-readable and unique identifier for a page.
 *
 * Underscores (`_`) are replaced by spaces (` `).
 *
 * A page title is not to be confused with a display title.
 * A display title is the preferred title associated with a wiki page (stored separately), with less restrictions than a page title.
 * @param sanitizedPageTitle
 * @see https://www.mediawiki.org/wiki/Manual:Page_title
 * @see https://www.mediawiki.org/wiki/Display_title
 * @returns
 * @example fromSanitizedPageTitleToPageTitle("Foo_bar") // "Foo bar"
 */
export const fromSanitizedPageTitleToPageTitle = (
  sanitizedPageTitle: string,
): string => {
  return sanitizedPageTitle.replaceAll("_", " ")
}
